/*
 ============================================================================
 Kaloth's BB4win Plugin Manager! - BBPlugManager
 ============================================================================
 
 This plugin takes care of all that messy plugins.rc editing and
 lets you load other plugins on the fly without restarting BB4win
 and download plugin updates from the dev's site.
 
 Blackbox for Windows is free software and so is this plugin
 released under the GNU General Public License (GPL version 2 or later).
  
 You may mess with it as you wish! (But keep it open source please :))
 
 ============================================================================
*/

#include "BBPlugManager.h"

extern "C"
{
	DLL_EXPORT int beginSlitPlugin(HINSTANCE hMainInstance, HWND hBBSlit);
	DLL_EXPORT int beginPluginEx(HINSTANCE hMainInstance, HWND hBBSlit);
	DLL_EXPORT int beginPlugin(HINSTANCE hMainInstance);
	DLL_EXPORT void endPlugin(HINSTANCE hMainInstance);
	DLL_EXPORT LPCSTR pluginInfo(int field);
}

/*
 ============================================================================
 Function Prototypes
 ============================================================================
 */

void ReadRCSettings();
void ReadPluginSettings();
void GetStyleSettings();
LRESULT CALLBACK WndProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam);
void ShowMyMenu(bool popup);

/*
 ============================================================================
 Global Data
 ============================================================================
 */

/*
 ============================================================================
 Plugin Functions
 ============================================================================
 */

// Load all plugins
void LoadPlugins(){
	int n;
	for(n = 0; n < MAX_PLUGINS; n++){
		if(plugs[n].inuse && plugs[n].loadOnStart){
			LoadPlugin(&plugs[n]);
		}
	}
}

// Unload all the plugins
void UnloadPlugins(){
	int n;
	for(n = 0; n < MAX_PLUGINS; n++){
		if(plugs[n].inuse && plugs[n].running){
			UnloadPlugin(&plugs[n]);
		}
	}
}

void pluginBroams(int n){
	char buffer[10000] = "";
	char minibuf[1000];
	
	if(plugs[n].inuse && plugs[n].running){
		if(plugs[n].Info != NULL && !STREQ(plugs[n].broams, "")){
			sprintf(minibuf, "%s\n\n%s", plugs[n].name, plugs[n].broams);
			strcat(buffer, minibuf);
		}
	}
	
	MessageBox(NULL, buffer, "Plugin Bro@ms...", MB_OK | MB_ICONINFORMATION);
}

void aboutPlugins(){
	int n;
	char buffer[1000] = "";
	char minibuf[100];
	for(n = 0; n < MAX_PLUGINS; n++){
		if(plugs[n].inuse && plugs[n].running){
			if(plugs[n].Info != NULL){
				sprintf(minibuf, "%s %s by %s (%s)\n", plugs[n].name, plugs[n].Info(PLUGIN_VERSION), plugs[n].Info(PLUGIN_AUTHOR), plugs[n].Info(PLUGIN_RELEASE));
			}else{
				sprintf(minibuf, "%s\n", plugs[n].path);
			}
			strcat(buffer, minibuf);
		}
	}
	
	MessageBox(NULL, buffer, "About Plugins...", MB_OK | MB_ICONINFORMATION);
}

void removePlugin(int n){
	// unload plugin.
	if(plugs[n].running)
		UnloadPlugin(&plugs[n]);
	// mark space as unused.
	plugs[n].inuse = false;
	
	//modify plugins.rc
	char buffer[MAX_PATH];
	strcpy(buffer, plugs[n].path);
	char *tmp = strstr(buffer, bbpath);
	strcpy(buffer, tmp + strlen(bbpath));
	
	FILE *file;
	file = fopen(plugpath, "r");
	if(file != NULL){
		// Get all lines but the one we want to remove.
		char lines[MAX_PLUGINS][MAX_PATH];
		char dat[MAX_PATH];
		int n = 0;
		while(fgets(dat, MAX_PATH, file) != NULL && n < MAX_PLUGINS){
			// if not the line we want to remove...
			if(strstr(dat, buffer) == NULL){
				strcpy(lines[n], dat);
				n++;
			}
			
			// Is this the end of the file?
			if(feof(file)){
				break;
			}
		}
		fclose(file);
		
		file = fopen(plugpath, "w");
		if(file != NULL){
			int i;
			for(i = 0; i < n; i++){
				fwrite(lines[i], strlen(lines[i]), 1, file);
			}
			fclose(file);
		}else{
			MessageBox(0,"Could not write to plugins.rc", szVersion,MB_OK | MB_ICONERROR | MB_TOPMOST);
		}
	}else{
		MessageBox(0,"Could not read from plugins.rc", szVersion,MB_OK | MB_ICONERROR | MB_TOPMOST);
	}
}
DWORD WINAPI CheckForUpdates(LPVOID lpParam){
	StartCogRotation();
	
	SendMessage(hwndBlackbox, BB_SETTOOLBARLABEL, 0, (LPARAM)"Downloading Update Information...");
	
	char path[MAX_PATH];
	sprintf(path, "%smetadata.txt", bbpath);
	int n;
	
	//Clear metafile...
	FILE *file = fopen(path, "w");
	if(file != NULL)
		fclose(file);
	
	//get plugin specific meta files.
	char urls[MAX_PLUGINS][MAX_PATH];
	bool found;
	int free;
	
	for(n = 0; n < MAX_PLUGINS; n++){
		if(plugs[n].inuse && !STREQ(plugs[n].URL, "")){
			found = false;
			for(int i = 0; i < MAX_PLUGINS; i++){
				if(STREQ(urls[i], "")){
					free = i;
				}else if(STREQ(urls[i], plugs[n].URL)){
					found = true;
					break;
				}
			}
			if(!found){
				DownloadFile_Merge(plugs[n].URL, path);
				strcpy(urls[free], plugs[n].URL);
			}
		}
	}
	
	//add the default on the end.
	DownloadFile_Merge(defaultUpdateURL, path);
	
	char cur_version[100];
	char cur_release[100];
	
	char buffer[5012];
	char version[100];
	char release[100];
	int total = 0;
	
	for(n = 0; n < MAX_PLUGINS; n++){
		if(plugs[n].inuse && plugs[n].running){
			ZeroMemory(version, 100); ZeroMemory(release, 100);
			ZeroMemory(cur_version, 100); ZeroMemory(cur_release, 100);
			
			sprintf(buffer, "%s.version:", plugs[n].name);
			strcpy(version, ReadString(path, buffer, "[No Info]"));
			sprintf(buffer, "%s.date:", plugs[n].name);
			strcpy(release, ReadString(path, buffer, "[No Info]"));
			
			if(plugs[n].Info != NULL){
				strcpy(cur_version, plugs[n].Info(PLUGIN_VERSION));
				strcpy(cur_release, plugs[n].Info(PLUGIN_RELEASE));
			}else{
				strcpy(cur_version, "");
				strcpy(cur_release, "");
			}
			
			CleanString(version); CleanString(release);
			CleanString(cur_version); CleanString(cur_release);
			
			if((!STREQ(version, cur_version) || !STREQ(release, cur_release))
			   && !STREQ(version, "[No Info]") && !STREQ(release, "[No Info]")
			   && !STREQ(version, "") && !STREQ(release, "")){
				plugs[n].updateavailable = true;
				total++;
			}else{
				plugs[n].updateavailable = false;
			}
		}
	}
	
	if(total){sprintf(buffer, "There are %d updates available.\n"
				  "\n"
				  "Use the \"Update Plugin\" menu\n"
				  "to choose the updates you wish\n"
				  "to install.", total);}
	else{sprintf(buffer, "There are no updates available.");}
	MessageBox(0, buffer, szVersion,MB_OK | MB_ICONINFORMATION | MB_TOPMOST);
	
	StopCogRotation();
	
	return 0;
}

DWORD WINAPI UpdatePlugin(LPVOID lpParam){
//void UpdatePlugin(int n){
	int n = PluginToUpdate;
	
	StartCogRotation();
	
	Plugin *plug = &plugs[n];
	
	char path[MAX_PATH];
	sprintf(path, "%smetadata.txt", bbpath);
	char buffer[5012];
	
	// We have the meta file, so process it!
	char cur_version[100];
	char cur_release[100];
	ZeroMemory(cur_version, 100); ZeroMemory(cur_release, 100);
	
	if(plugs[n].Info != NULL){
		strcpy(cur_version, plug->Info(PLUGIN_VERSION));
		strcpy(cur_release, plug->Info(PLUGIN_RELEASE));
	}else{
		strcpy(cur_version, "");
		strcpy(cur_release, "");
	}
	
	char version[100];
	char release[100];
	char author[100];
	char mods[4096];
	char descrip[4096];
	char files[MAX_PATH];
	char archive[4096];
	
	ZeroMemory(version, 100); ZeroMemory(release, 100);
	
	// Read meta file...
	sprintf(buffer, "%s.version:", plug->name);
	strcpy(version, ReadString(path, buffer, "[No Info]"));
	sprintf(buffer, "%s.date:", plug->name);
	strcpy(release, ReadString(path, buffer, "[No Info]"));
	sprintf(buffer, "%s.author:", plug->name);
	strcpy(author, ReadString(path, buffer, "[No Info]"));
	sprintf(buffer, "%s.changes:", plug->name);
	strcpy(mods, ReadString(path, buffer, "[No Info]"));
	sprintf(buffer, "%s.files:", plug->name);
	strcpy(files, ReadString(path, buffer, "[No Info]"));
	sprintf(buffer, "%s.information:", plug->name);
	strcpy(descrip, ReadString(path, buffer, "[No Info]"));
	sprintf(buffer, "%s.archive:", plug->name);
	strcpy(archive, ReadString(path, buffer, "[No Info]"));
	
	// Clean up the strings...
	CleanString(version); CleanString(release);
	CleanString(cur_version); CleanString(cur_release);
	CleanString(files); CleanString(archive);
	
	// Tell user appropriate info...
	if(STREQ(version, "[No Info]") || STREQ(version, "")){
		MessageBox(0, "No update information was available", szVersion, MB_OK | MB_ICONERROR | MB_TOPMOST);
		plug->updateavailable = false;
		StopCogRotation();
		PluginToUpdate = -1;
		return 0;
	}else if(STREQ(version, cur_version) && STREQ(release, cur_release)){
		MessageBox(0, "You already have the online version of this plugin", szVersion, MB_OK | MB_ICONERROR | MB_TOPMOST);
		plug->updateavailable = false;
		StopCogRotation();
		PluginToUpdate = -1;
		return 0;
	}else{
		sprintf(buffer, "Name: \t\t%s\n"
				"Current Version: \t%s\n"
				"Date: \t\t%s\n"
				"\n"
				"Online Version: \t%s\n"
				"Date: \t\t%s\n"
				"Author: \t\t%s\n"
				"\n"
				"Description...\n%s\n"
				"\n"
				"Changes...\n%s\n"
				"\n"
				"Do you want to download this update?",
				plug->name, cur_version, cur_release, version, release, author, descrip, mods);
		if(IDYES == MessageBox(0, buffer, szVersion, MB_YESNO | MB_ICONINFORMATION| MB_TOPMOST)){
			// If no incremental update files so grab the archive!
			if(STREQ(files, "[No Info]") || STREQ(files, "")){
				if(!STREQ(archive, "[No Info]") && !STREQ(archive, "")){
					strcpy(files, archive);
				}else{
					// No file info available -> tell user, then quit.
					MessageBox(0, "No file information was provided!\nPlease contact the plugin's developer.", szVersion, MB_OK | MB_ICONERROR | MB_TOPMOST);
					StopCogRotation();
					PluginToUpdate = -1;
					return 0;
				}
			}
			
			//Unload the current plugin version...
			sprintf(buffer, "@BBPlugManagerInternal TogglePlugin %d", n);
			SendMessage(hwndPlugin, BB_BROADCAST, 0, (LPARAM)buffer);
			int time = GetTickCount();
			while(plug->running){
				if((GetTickCount() - time) > TIME_OUT){
					// Report errors then abort...
					MessageBox(0, "A Problem occurred when trying\nto reload the plugin.", szVersion, MB_OK | MB_ICONERROR | MB_TOPMOST);
					StopCogRotation();
					PluginToUpdate = -1;
					return 0;
				}
			}
			
			// Get the update file...
			char *file;
			file = strtok(files, " ");
			while(file != NULL){
				CleanString(file);
				
				char localfile[MAX_PATH];
				strcpy(localfile, getURLFileName(file));
				char filenm[MAX_PATH];
				sprintf(buffer, "Downloading %s...", localfile);
				SendMessage(hwndBlackbox, BB_SETTOOLBARLABEL, 0, (LPARAM)buffer);
				sprintf(filenm, "%s%s", getFileDir(plug->path), localfile);
				CleanString(filenm);
				
				// Retrieve the update file...
				if(!DownloadFile(file, filenm)){
					// Report errors then abort...
					MessageBox(0, "Could not complete download", szVersion, MB_OK | MB_ICONERROR | MB_TOPMOST);
					StopCogRotation();
					PluginToUpdate = -1;
					return 0;
				}
				
				file = strtok(NULL, " ");
			}
			plug->updateavailable = false;
			// Tell user process has finished sucessfully!
			// Load the new plugin version...
			sprintf(buffer, "@BBPlugManagerInternal TogglePlugin %d", n);
			SendMessage(hwndPlugin, BB_BROADCAST, 0, (LPARAM)buffer);
			
			MessageBox(0, 	"Update Procedure Complete.\n"
					"\n"
					"If the download included a zip file you\n"
					"may have to check that the files where\n"
					"extracted to the right place ;P"
				   , szVersion, MB_OK | MB_ICONINFORMATION | MB_TOPMOST);
		}
	}
	StopCogRotation();
	PluginToUpdate = -1;
	return 0;
}

void MarkToggle(int num, bool mark, bool slitaid){
	char buffer[1000], buffer2[1000];
	
	//make plug path string...
	strcpy(buffer2, strlwr(plugs[num].path));
	char *tmp = strstr(buffer2, bbpath);
	tmp += strlen(bbpath);
	strcpy(buffer2, tmp);
	
	//if(mark){sprintf(buffer, "!!%s\n", buffer2);}
	//else{sprintf(buffer, "!%s\n", buffer2);}
	
	sprintf(buffer, "%s%s%s\n", mark?"!":"", slitaid?"&":"", buffer2);
	
	//modify plugins.rc
	FILE *file;
	file = fopen(plugpath, "r");
	if(file != NULL){
		// Get all lines but the one we want to change.
		char lines[100][MAX_PATH];
		char dat[MAX_PATH];
		int n = 0;
		while(fgets(dat, MAX_PATH, file) != NULL && n < 100){
			// if not the line we want to change...
			strcpy(dat, strlwr(dat));
			if(strstr(dat, buffer2) == NULL){
				strcpy(lines[n], dat);
				n++;
			}else{
				strcpy(lines[n], buffer);
				n++;
			}
			
			// Is this the end of the file?
			if(feof(file)){
				break;
			}
		}
		fclose(file);
		
		file = fopen(plugpath, "w");
		if(file != NULL){
			int i;
			for(i = 0; i < n; i++){
				fwrite(lines[i], strlen(lines[i]), 1, file);
			}
			fclose(file);
		}else{
			MessageBox(0,"Could not write to plugins.rc", szVersion,MB_OK | MB_ICONERROR | MB_TOPMOST);
		}
	}else{
		MessageBox(0,"Could not read from plugins.rc", szVersion,MB_OK | MB_ICONERROR | MB_TOPMOST);
	}
}

void addPlugin(){
	char filenm[100];
	strcpy(filenm, "");
	
	OPENFILENAME ofn;
	ofn.lStructSize = sizeof(OPENFILENAME);
	ofn.hwndOwner = NULL;
	ofn.hInstance = NULL;
	ofn.lpstrFilter = "Dll Files\0*.dll\0\0";
	ofn.lpstrCustomFilter = NULL;
	ofn.nMaxCustFilter = NULL;
	ofn.nFilterIndex = NULL;
	ofn.lpstrFile = filenm;
	ofn.nMaxFile = 100;
	ofn.lpstrFileTitle = NULL;
	ofn.nMaxFileTitle = NULL;
	ofn.lpstrInitialDir = NULL; // start in users default folder.
	ofn.lpstrTitle = "BBPlugManager - Select a .dll";
	ofn.Flags = NULL;
	ofn.nFileOffset = 0;
	ofn.nFileExtension = 0;
	ofn.lpstrDefExt = NULL;
	ofn.lCustData = NULL;
	ofn.lpTemplateName = "";
	ofn.pvReserved = NULL;
	ofn.dwReserved = 0;
	ofn.FlagsEx = 0;
	
	if(GetOpenFileName(&ofn)){
		int n, place = -1;
		bool found = false;
		
		fixPath(filenm);
		strcpy(filenm, strlwr(filenm));
		
		for(n = 0; n < MAX_PLUGINS; n++){
			if(STREQ(plugs[n].path, filenm)){
				found = true;
			}
			
			if(!plugs[n].inuse && place == -1){
				place = n;
			}
		}
		
		// If now room for more plugins tell the usr...
		if(place >= MAX_PLUGINS || place == -1){
			MessageBox(0,"Cannot add plugin!\n"
				     "\n"
				     "You have reached the maximum\n"
				     "number of plugins (50).\n"
				     "\nPlease remove some first ;)\n", szVersion,MB_OK | MB_ICONERROR | MB_TOPMOST);
			return;
		}else{
			n = place;
		}
		
		// If user trying to add more than one instance of a plugin
		// ask them if they realy want to...
		if(found){
			if(IDNO == MessageBox(0, "You have already added this plugin.\n"
						  "Are you sure you want to add another\n"
						  "instance of it?",
						szVersion, MB_YESNO | MB_ICONINFORMATION| MB_TOPMOST)){
				return;
			}
		}
		
		// Make sure the plugin is in the right place...
		if(strstr(filenm, bbpath) == NULL){
			MessageBox(0,"Using plugins from a directory outside\n"
				     "the standard folder can cause serious\n"
				     "problems. BBPlugManager does not allow\n"
				     "you to do this. (Sorry!)", szVersion,MB_OK | MB_ICONERROR | MB_TOPMOST);
			return;
		}
		
		// make an initial link to the plugin dll.
		if(linkPlugin(filenm, &plugs[n])){
			LoadPlugin(&plugs[n]); // start the plugin rolling.
			
			char buffer[1000];
			char *tmp = strstr(filenm, bbpath);
			strcpy(filenm, tmp + strlen(bbpath));
			sprintf(buffer, "\n!%s", filenm);
			
			FILE *file;
			file = fopen(plugpath, "a+");
			if(file != NULL){
				fwrite(buffer, strlen(buffer), 1, file);
				fclose(file);
			}else{
				sprintf(buffer, "Could not find %s", plugRCFileName);
				MessageBox(0, buffer, szVersion,MB_OK | MB_ICONERROR | MB_TOPMOST);
			}
		}
	}
}

/*
 ============================================================================
 Misc Utility Functions
 ============================================================================
 */

void set_window_modes(void)
{
	if (hidden && pluginToggle)
	{
		ShowWindow(hwndPlugin, SW_HIDE);
	}
	else
	{
		if(usingWin2kXP){
			if(!inSlit){
				// Add the WS_EX_LAYERED atribute to the window.
				SetWindowLong(hwndPlugin, GWL_EXSTYLE , GetWindowLong(hwndPlugin, GWL_EXSTYLE) | WS_EX_LAYERED);
				// Make it transparent...
				SetLayeredWindowAttributes(	hwndPlugin,
								TRANSCOLOUR,
								(transparencyEnabled) ? (unsigned char)transparencyAlpha : 255,
								LWA_COLORKEY | LWA_ALPHA);
			}else{
				// Remove the WS_EX_LAYERED atribute to the window, cos we in di slit!
				SetWindowLong(hwndPlugin, GWL_EXSTYLE, GetWindowLong(hwndPlugin, GWL_EXSTYLE) & ~WS_EX_LAYERED);
			}
		}
		
		SetWindowPos(
			hwndPlugin,
			alwaysOnTop ? HWND_TOPMOST : HWND_NOTOPMOST,
			xpos, ypos, width, height, SWP_NOACTIVATE|SWP_SHOWWINDOW
			);
		
		InvalidateRect(hwndPlugin, NULL, false);
	}
}

/*
 ============================================================================
 Plugin Start, End and Info Methods + Slit Support! (YAY)
 ============================================================================
 */

int beginSlitPlugin(HINSTANCE hMainInstance, HWND hBBSlit)
{
	inSlit = true;
	hSlit = hBBSlit;
	
	// Start the plugin like normal now..
	int res = beginPlugin(hMainInstance);
	
	// Are we to be in the Slit?
	if (inSlit && hSlit){
		// Yes, so Let's let BBSlit know.
		SendMessage(hSlit, SLIT_ADD, NULL, (LPARAM)hwndPlugin);
	}
	
	return res;
}

int beginPluginEx(HINSTANCE hMainInstance, HWND hBBSlit)
{
	return beginSlitPlugin(hMainInstance, hBBSlit);
}

int beginPlugin(HINSTANCE hPluginInstance){
	WNDCLASS wc;
	
	GetBlackboxPath(bbpath, MAX_PATH);
	strcpy(bbpath, strlwr(bbpath));
	
	ZeroMemory(&osvinfo, sizeof(osvinfo));
	osvinfo.dwOSVersionInfoSize = sizeof (osvinfo);
	GetVersionEx(&osvinfo);
	usingWin2kXP =
		osvinfo.dwPlatformId == VER_PLATFORM_WIN32_NT
		&& osvinfo.dwMajorVersion > 4;

	hwndBlackbox = GetBBWnd();
	hInstance = hPluginInstance;

	// Register the window class...
	ZeroMemory(&wc,sizeof(wc));
	wc.lpfnWndProc = WndProc;           // our window procedure
	wc.hInstance = hPluginInstance;     // hInstance of .dll
	wc.lpszClassName = szAppName;       // our window class name
	wc.hCursor = LoadCursor(NULL, IDC_ARROW);
	wc.style = CS_DBLCLKS;

	if (!RegisterClass(&wc)) 
	{
		MessageBox(hwndBlackbox,
			"Error registering window class", szVersion,
				MB_OK | MB_ICONERROR | MB_TOPMOST);
		return 1;
	}
	
	// Get plugin and style settings...
	ReadRCSettings();
	ReadPluginSettings();
	GetStyleSettings();

	// Create the window...
	if (!(hwndPlugin = CreateWindowEx(
		usingWin2kXP ? WS_EX_TOOLWINDOW | WS_EX_LAYERED : WS_EX_TOOLWINDOW, // window style
		szAppName,                      // our window class name
		NULL,                           // NULL -> does not show up in task manager!                
		WS_POPUP
		| WS_CLIPCHILDREN
		| WS_CLIPSIBLINGS
		| WS_VISIBLE,             // window parameters
		xpos,                           // x position
		ypos,                           // y position
		width,                          // window width
		height,                         // window height
		NULL,                           // parent window
		NULL,                           // no menu
		hPluginInstance,                // hInstance of .dll
		NULL)))
	{
		MessageBox(0,
			"Error creating window", szVersion,
				MB_OK | MB_ICONERROR | MB_TOPMOST);
		return 1;
	}
	
	// Startup the plugins.
	LoadPlugins();
	
	// Apply Global Transparency...
	if(usingWin2kXP){hook();}
	
	return 0;
}

void endPlugin(HINSTANCE hPluginInstance){
	if(usingWin2kXP){unhook();}
	if(clipmenu){
		EndMenuClipper();
	}
	
	// Jump outa da slit!
	if(inSlit && hSlit){
		// We are in the slit, so lets unload and get out..
		SendMessage(hSlit, SLIT_REMOVE, NULL, (LPARAM)hwndPlugin);
		
		// Here you can move to where ever you want ;)
		SetWindowPos(hwndPlugin, NULL, xpos, ypos, 0, 0, SWP_NOZORDER | SWP_NOSIZE);
		inSlit = false;
	}
	
	// Destroy the window...
	DestroyWindow(hwndPlugin);
	
	// Unregister window classes...
	UnregisterClass(szAppName, hPluginInstance);
	
	// End all the plugins.
	UnloadPlugins();
}

LPCSTR pluginInfo(int field){
	// pluginInfo is used by Blackbox for Windows to fetch information about
	// a particular plugin. At the moment this information is simply displayed
	// in an "About loaded plugins" MessageBox, but later on this will be
	// expanded into a more advanced plugin handling system. Stay tuned! :)
	switch (field)
	{
		case PLUGIN_NAME:
			return szAppName;       // Plugin name
		case PLUGIN_VERSION:
			return szInfoVersion;   // Plugin version
		case PLUGIN_AUTHOR:
			return szInfoAuthor;    // Author
		case PLUGIN_RELEASE:
			return szInfoRelDate;   // Release date, preferably in yyyy-mm-dd format
		case PLUGIN_LINK:
			return szInfoLink;      // Link to author's website
		case PLUGIN_EMAIL:
			return szInfoEmail;     // Author's email
		default:
			return szVersion;       // Fallback: Plugin name + version, e.g. "MyPlugin 1.0"
	}
}

/*
 ============================================================================
 RC & Settings Functions
 ============================================================================
 */

void set_setting(bool *flag, const char *toolbar_message, const char *rc_string)
{
	// send a display sting to the toolbar
	char msg[80];
	sprintf(msg, "BBPlugManager -> %s %sabled!",
		toolbar_message, *flag ? "en" : "dis");
	SendMessage(hwndBlackbox, BB_SETTOOLBARLABEL, 0, (LPARAM)msg);

	// write the new setting to the rc - file
	WriteBool(rcpath, rc_string, *flag);

	// apply new settings
	set_window_modes();

	// and update the menu checkmarks
	ShowMyMenu(false);
}

 
void toggle_setting(bool *flag, const char *toolbar_message, const char *rc_string)
{
	// toggle the setting
	*flag = false == *flag;

	// send a display sting to the toolbar
	char msg[80];
	sprintf(msg, "BBPlugManager -> %s %sabled!",
		toolbar_message, *flag ? "en" : "dis");
	SendMessage(hwndBlackbox, BB_SETTOOLBARLABEL, 0, (LPARAM)msg);

	// write the new setting to the rc - file
	WriteBool(rcpath, rc_string, *flag);

	// apply new settings
	set_window_modes();

	// and update the menu checkmarks
	ShowMyMenu(false);
}

void ReadPluginSettings(){
	// Read from plugins.rc and plugin paths begining with '!'
	// Note: you can still comment out plugins with '#'
	
	strcpy(plugpath, BBPM_plugrcPath());
	
	FILE *file;
	file = fopen(plugpath, "rt");
	if(file != NULL){
		int n = 0, i, len;
		char buffer[MAX_PATH], dat[MAX_PATH], *ptr;
		while(fgets(dat, MAX_PATH, file) != NULL && n < MAX_PLUGINS){
			if(dat[0] != '\r' && dat[0] != '\n'){
				ptr = dat;
				if(dat[0] == '!'){plugs[n].loadOnStart = false; ptr++;}
				else{plugs[n].loadOnStart = true;}
				if(*ptr == '&'){plugs[n].slitaid = true; ptr++;}
				else{plugs[n].slitaid = false;}
				len = strlen(ptr);
				strcpy(buffer, strtok(ptr, "\r\n\t "));
				char path[MAX_PATH];
				ZeroMemory(path, MAX_PATH);
				strcat(path, buffer);
				fixPath(path);
				strcpy(path, strlwr(path));
				CleanString(path);
				if(!STREQ(buffer, "")){
					linkPlugin(path, &plugs[n]);
					n++;
				}
			}
			
			// Is this the end of the file?
			if(feof(file)){
				break;
			}
		}
		
		fclose(file);
	}else{
		char buffer[100];
		sprintf(buffer, "Could not find %s", plugRCFileName);
		MessageBox(0, buffer, szVersion,MB_OK | MB_ICONERROR | MB_TOPMOST);
	}
}

void ReadRCSettings(){
	char defaultpath[MAX_PATH]; int nLen, i;
	// First we look for the config file in the same folder as the plugin...
	GetModuleFileName(hInstance, rcpath, sizeof(rcpath));
	for (i=0;;i++)
	{
		nLen = strlen(rcpath); while (nLen && rcpath[nLen-1] != '\\') nLen--;
		// ...check the two possible filenames 'examplerc' and 'example.rc'...
		strcpy(rcpath+nLen, "BBPlugManagerrc");  if (FileExists(rcpath)) break;
		strcpy(rcpath+nLen, "BBPlugManager.rc"); if (FileExists(rcpath)) break;
		if (0 == i)
		{   //...if not found aside the plugin, we keep this as the default path...
			strcpy(defaultpath, rcpath);
			//...and try the Blackbox directory.
			GetBlackboxPath(rcpath, sizeof(rcpath));
			continue;
		}
		//if again no config file was found, we use the default path...
		strcpy(rcpath, defaultpath);
		break; // ...and quit the loop
	}
	// If a config file was found we read the plugin settings from it...
	// ...if not, the ReadXXX functions give us just the defaults.
	
	xpos = ReadInt(rcpath, "BBPlugManager.plugin.x:", 10);
	ypos = ReadInt(rcpath, "BBPlugManager.plugin.y:", 10);
	if (xpos >= GetSystemMetrics(SM_CXSCREEN)) xpos = 10;
	if (ypos >= GetSystemMetrics(SM_CYSCREEN)) ypos = 10;
	transparencyEnabled 	= ReadBool(rcpath, "BBPlugManager.plugin.transparency.enabled:", false);
	transparencyAlpha   	= ReadInt(rcpath,  "BBPlugManager.plugin.transparency.alpha:", 60);
	alwaysOnTop        	= ReadBool(rcpath, "BBPlugManager.plugin.alwaysontop:", true);
	snapWindow          	= ReadBool(rcpath, "BBPlugManager.plugin.snapwindow:", true);
	pluginToggle        	= ReadBool(rcpath, "BBPlugManager.plugin.togglewithplugins:", true);
	strcpy(defaultUpdateURL, ReadString(rcpath, "BBPlugManager.plugin.defaultupdateURL:", ""));
	strcpy(plugRCFileName, ReadString(rcpath, "BBPlugManager.plugin.PluginsRCFile:", "BBPM_Plugins.rc"));
	transback	   	= ReadBool(rcpath, "BBPlugManager.plugin.transback.enabled:", false);
	inSlit	   		= ReadBool(rcpath, "BBPlugManager.plugin.inSlit:", false);
	globalTransparency 	= ReadBool(rcpath, "BBPlugManager.plugin.GlobalTransparency:", false);
	globalTransAlpha	=  ReadInt(rcpath, "BBPlugManager.plugin.GlobTransAlpha:", 0);
	globalSticky 		= ReadBool(rcpath, "BBPlugManager.plugin.GlobalSticky:", false);
	clipmenu 		= ReadBool(rcpath, "BBPlugManager.plugin.menuclipper.enabled:", false);
	
	width = ReadInt(rcpath, "BBPlugManager.plugin.width:", 25);
	height = ReadInt(rcpath, "BBPlugManager.plugin.height:", 25);;
	
	char buffer[MAX_PATH];
	for(int n = 0; n < 50; n++){
		sprintf(buffer, "BBPlugManager.plugin.transClass%d:", n);
		strcpy(classes[n], ReadString(rcpath, buffer, ""));
		sprintf(buffer, "BBPlugManager.plugin.stickyClass%d:", n);
		strcpy(sticky[n], ReadString(rcpath, buffer, ""));
	}
	
	if(clipmenu){
		StartMenuClipper();
	}
}

/*
 ============================================================================
 GUI Functions
 ============================================================================
 */

void ShowMyMenu(bool popup)
{
	Menu *pMenu, *pSub, *plugMenu, *remMenu, *transremMenu, *stickyremMenu, *transAdjust;
	Menu *transcontmenu, *stickycontmenu, *plugbroam, *menuclipmenu, *aidMenu;
	char buffer[1000], buffer2[1000];
	// We create the main plugin menu.
	// Each menu or submenu has an unique ID-string, which identifies it
	// for later updates.

	pMenu = MakeNamedMenu("BBPlugManager", "BBPlugManager_IDMain", popup);

	// --------------------------------------------------------------
	// ... and the configuration options menu, note the "internal" bro@ms!...
	pSub = MakeNamedMenu("Configuration", "BBPlugManager_IDConfig", popup);
	MakeMenuItem(pSub, "Docked to Slit", "@BBPlugManagerInternal SlitToggle", inSlit);
	MakeMenuItem(pSub, "Always On Top", "@BBPlugManagerInternal AlwaysOnTop", alwaysOnTop);
	MakeMenuItem(pSub, "Snap Window To Edge", "@BBPlugManagerInternal SnapWindowToEdge", snapWindow);
	MakeMenuItem(pSub, "Toggle With Plugins", "@BBPlugManagerInternal ToggleWithPlugins", pluginToggle);
	if(usingWin2kXP)MakeMenuItem(pSub, "Transparency Enabled", "@BBPlugManagerInternal TransparencyEnabled", transparencyEnabled);
	// insert an item to adjust an integer value. Implemented in bblean for now
	MakeMenuItemInt(pSub, "Alpha Value", "@BBPlugManagerInternal TransparencyAlpha", transparencyAlpha, 0, 255);
	if(usingWin2kXP)MakeMenuItem(pSub, "Transparent Back Enabled", "@BBPlugManagerInternal TransbackEnabled", transback); // TRANSBACK NEEDS FINISHING!
	MakeMenuNOP(pSub, NULL);
	if(usingWin2kXP){
		int n;
		// Transparency Controller...
		transcontmenu = MakeNamedMenu("Transparency Controller", "BBPlugManager_IDTransMenu", popup);
			MakeMenuItem(transcontmenu, "Transparency Controller Enabled", "@BBPlugManagerInternal GlobalTransEnabled", globalTransparency);
			transAdjust = MakeNamedMenu("Transparency Controller Alpha", "BBPlugManager_IDTransAdjust", popup);
			for(n = 0; n <= 255; n += 5){
					sprintf(buffer, "@BBPlugManagerInternal GlobalTransAdjust %d", n);
					sprintf(buffer2, "%d", n);
					MakeMenuItem(transAdjust, buffer2, buffer, n == globalTransAlpha);
			}
			MakeSubmenu(transcontmenu, transAdjust, "Transparency Controller Alpha");
			MakeMenuItem(transcontmenu, "Add Transparency Window Class", "@BBPlugManagerInternal GlobalTransAdd", false);
			transremMenu = MakeNamedMenu("Remove Window Class", "BBPlugManager_IDTransClasses", popup);
			for(n = 0; n < MAX_CLASSES; n++){
				if(strcmp(classes[n], "") != 0){
					sprintf(buffer, "@BBPlugManagerInternal GlobalTransRem %d", n);
					MakeMenuItem(transremMenu, classes[n], buffer, false);
				}
			}
			MakeSubmenu(transcontmenu, transremMenu, "Remove Window Class");
		MakeSubmenu(pSub, transcontmenu, "Transparency Controller");
		
		// Sticky Controller...
		stickycontmenu = MakeNamedMenu("Sticky Controller", "BBPlugManager_IDStickyMenu", popup);
			MakeMenuItem(stickycontmenu, "Sticky Controller Enabled", "@BBPlugManagerInternal GlobalStickyEnabled", globalSticky);
			MakeMenuItem(stickycontmenu, "Add Sticky Window Class", "@BBPlugManagerInternal GlobalStickyAdd", false);
			stickyremMenu = MakeNamedMenu("Remove Window Class", "BBPlugManager_IDStickyClasses", popup);
			for(n = 0; n < MAX_CLASSES; n++){
				if(strcmp(sticky[n], "") != 0){
					sprintf(buffer, "@BBPlugManagerInternal GlobalStickyRem %d", n);
					MakeMenuItem(stickyremMenu, sticky[n], buffer, false);
				}
			}
			MakeSubmenu(stickycontmenu, stickyremMenu, "Remove Window Class");
		MakeSubmenu(pSub, stickycontmenu, "Sticky Controller");
		
		// Menu Clipper...
		menuclipmenu = MakeNamedMenu("Menu Clipper", "BBPlugManager_IDMenuClipper", popup);
			MakeMenuItem(menuclipmenu, "Menu Clipper Enabled", "@BBPlugManagerInternal MenuClipperEnabled", clipmenu);
		MakeSubmenu(pSub, menuclipmenu, "Menu Clipper");
	}
	
	// ...and add it to the main plugin menu
	MakeSubmenu(pMenu, pSub, "Configuration");
	MakeMenuNOP(pMenu, NULL);
	
	// The "Toggle Plugins" sub-menu...
	plugMenu = MakeNamedMenu("Toggle Plugins", "BBPlugManager_IDPlugins", popup);
	int n;
	for(n = 0; n < MAX_PLUGINS; n++){
		if(plugs[n].inuse){
			sprintf(buffer, "@BBPlugManagerInternal TogglePlugin %d", n);
			MakeMenuItem(plugMenu, plugs[n].name, buffer, plugs[n].running);
		}
	}
	MakeSubmenu(pMenu, plugMenu, "Toggle Plugins");
	
	MakeMenuItem(pMenu, "Add Plugin", "@BBPlugManagerInternal AddPlugin", false);
	
	// The "Remove Plugin" sub-menu...
	remMenu = MakeNamedMenu("Remove Plugin", "BBPlugManager_IDRemove", popup);
	for(n = 0; n < MAX_PLUGINS; n++){
		if(plugs[n].inuse){
			sprintf(buffer, "@BBPlugManagerInternal RemovePlugin %d", n);
			MakeMenuItem(remMenu, plugs[n].name, buffer, false);
		}
	}
	MakeSubmenu(pMenu, remMenu, "Remove Plugin");
	
	MakeMenuNOP(pMenu, NULL);
	MakeMenuItem(pMenu, "Check For Updates", "@BBPlugManagerInternal CheckForUpdates", false);
	// The "Update Plugin" sub-menu...
	remMenu = MakeNamedMenu("Update Plugin", "BBPlugManager_IDUpdate", popup);
	for(n = 0; n < MAX_PLUGINS; n++){
		if(plugs[n].inuse && plugs[n].running && plugs[n].updateavailable){
			sprintf(buffer, "@BBPlugManagerInternal UpdatePlugin %d", n);
			MakeMenuItem(remMenu, plugs[n].name, buffer, false);
		}
	}
	MakeSubmenu(pMenu, remMenu, "Update Plugin");
	
	MakeMenuNOP(pMenu, NULL);
	// The "Slit Aid" sub-menu...
	aidMenu = MakeNamedMenu("Slit Aid", "BBPlugManager_IDSlitAid", popup);
	for(n = 0; n < MAX_PLUGINS; n++){
		if(plugs[n].inuse && plugs[n].running && plugs[n].slitaidable){
			sprintf(buffer, "@BBPlugManagerInternal ToggleSlitAid %d", n);
			MakeMenuItem(aidMenu, plugs[n].name, buffer, plugs[n].slitaid);
		}
	}
	MakeSubmenu(pMenu, aidMenu, "Slit Aid");
	
	MakeMenuNOP(pMenu, NULL);
	MakeMenuItem(pMenu, "About Plugins", "@BBPlugManagerInternal AboutPlugins", false);
	
	// The "Plugin Bro@ms" sub-menu...
	plugbroam = MakeNamedMenu("Plugin Bro@ms", "BBPlugManager_IDPluginBroamMenu", popup);
	char bigbuf[10000]; char *ptr;
	for(n = 0; n < MAX_PLUGINS; n++){
		if(plugs[n].inuse && plugs[n].running && !STREQ(plugs[n].broams, "")){
			Menu *plugbroams;
			sprintf(buffer2, "BBPlugManager_IDPluginBroams %s", plugs[n].name);
			plugbroams = MakeNamedMenu(plugs[n].name, buffer2, popup);
				ZeroMemory(bigbuf, 10000);
				strcpy(bigbuf, plugs[n].broams);
				ptr = strtok(bigbuf, "\n\0");
				while(ptr != NULL){
					strcpy(buffer2, ptr);
					sprintf(buffer, "@BBPlugManagerInternal SendPluginBroam %s", buffer2);
					MakeMenuItem(plugbroams, buffer2, buffer, false);
					ptr = strtok(NULL, "\n\0");
				}
			MakeSubmenu(plugbroam, plugbroams, plugs[n].name);
		}
	}
	MakeSubmenu(pMenu, plugbroam, "Plugin Bro@ms");
	
	MakeMenuNOP(pMenu, NULL);
	// View the readme using the default editor...
	MakeMenuItem(pMenu, "View Readme", "@BBPlugManagerInternal Edit Readme.txt", false);
	// ...an "About" box is always nice, even though we now have the regular plugin info function...
	MakeMenuItem(pMenu, "About BBPlugManager", "@BBPlugManagerInternal About", false);
	// Edit the RC-File, using the default Blackbox Editor
	MakeMenuItem(pMenu, "Edit BBPlugManager.rc", "@BBPlugManagerInternal Edit BBPlugManager.rc", false);
	
	// And finally, we show the menu...
	ShowMenu(pMenu);
}

void DrawWindow(HWND hwnd){
	PAINTSTRUCT ps; RECT r;
	
	//get the screen-buffer
	HDC hdc = BeginPaint(hwnd, &ps);
	
	//get window rectangle
	GetClientRect(hwnd, &r);
	
	//to prevent flicker of the display, we draw to memory first,
	//then put it on screen in one single operation. This is like this:
	
	//first get a new 'device context'
	HDC buf = CreateCompatibleDC(NULL);
	
	//then create a buffer in memory with the window size
	HBITMAP bufbmp = CreateCompatibleBitmap(hdc, r.right, r.bottom);
	
	//select it into the DC and store, whatever there was before.
	HGDIOBJ otherbmp = SelectObject(buf, bufbmp);
	
	// use new api
	// Paint background according to the current style on the bitmap,
	// using the default border from style (true)
	if(!transback || inSlit){
		MakeStyleGradient(buf, &r, &myStyleItem, true);
	}else{
		HBRUSH hbbrush = CreateSolidBrush(TRANSCOLOUR);
		FillRect(buf, &r, hbbrush);
		DeleteObject(hbbrush);
	}
	
	//Draw the cog...
	DrawCog(buf, (r.right - r.left) >> 1, (r.bottom - r.top) >> 1,
		(width < height) ? (width-10)>>1 : (height-10)>>1);
	
	//now copy the drawing from the paint buffer to the window...
	BitBlt(hdc,
		ps.rcPaint.left,
		ps.rcPaint.top,
		ps.rcPaint.right  - ps.rcPaint.left,
		ps.rcPaint.bottom - ps.rcPaint.top,
		buf,
		ps.rcPaint.left,
		ps.rcPaint.top,
		SRCCOPY
		);
		
	//restore the first previous whatever to the dc,
	//get in exchange back our bitmap, and delete it.
	DeleteObject(SelectObject(buf, otherbmp));
	
	//delete the memory - 'device context'
	DeleteDC(buf);
	
	//done
	EndPaint(hwnd, &ps);
}

LRESULT CALLBACK WndProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	char szTemp[MAX_PATH];

	// Blackbox messages we want to "subscribe" to:
	// BB_RECONFIGURE -> Sent when changing style and on reconfigure
	// BB_BROADCAST -> Broadcast message (bro@m)
	static int msgs[] = { BB_RECONFIGURE, BB_BROADCAST, 0};
	
	switch (message)
	{       
		// ==========
		case WM_CREATE:
			hwndPlugin = hwnd;
		
			// Register to receive Blackbox messages...
			SendMessage(hwndBlackbox, BB_REGISTERMESSAGE, (WPARAM)hwnd, (LPARAM)msgs);
			
			// make the window visible on all worspaces
			MakeSticky(hwnd);
			
			set_window_modes();
			break;

		// ==========
		case WM_DESTROY:
			//remove us from the sticky-windows-list
			RemoveSticky(hwnd);

			// Unregister Blackbox messages...
			SendMessage(hwndBlackbox, BB_UNREGISTERMESSAGE, (WPARAM)hwnd, (LPARAM)msgs);
			break;
		
		// ==========
		// If Blackbox sends a reconfigure message, load the new style settings and
		// update the window...

		case BB_RECONFIGURE:
			EnumWindows((WNDENUMPROC)EnumWindowsOpaque, NULL);
			UnloadPlugins();
			ReadRCSettings();
			GetStyleSettings();
			set_window_modes();
			LoadPlugins();
			EnumWindows((WNDENUMPROC)EnumWindowTransparencyInit, NULL);
			break;

		// ==========
		// Window redraw processing...
		case WM_PAINT:
		{
			DrawWindow(hwnd);
			break;
		}

		// ==========
		// Broadcast messages (bro@m -> the bang killah! :D <vbg>)
		case BB_BROADCAST:
		{
			const char *msg_string = (LPCSTR)lParam;
			
			// First we check for the two "global" bro@ms,
			if (!stricmp(msg_string, "@BBShowPlugins"))
			{
				// @BBShowPlugins: Show window (global bro@m)
				if(!inSlit){
					hidden = false;
					set_window_modes();
				}
				break;
			}
			if (!stricmp(msg_string, "@BBHidePlugins"))
			{
				// @BBHidePlugins: Hide window (global bro@m)
				if(!inSlit){
					hidden = true;
					set_window_modes();
				}
				break;
			}

			// ==========
			// Finally, we check for our "internal" bro@m, which is used for
			// all plugin internal commands, for example the menu commands...

			static const char Example_Broam_ID[] = "@BBPlugManagerInternal";

			if (!strnicmp(msg_string, Example_Broam_ID, sizeof Example_Broam_ID - 1))
			{
				msg_string += sizeof Example_Broam_ID - 1;
				while (*msg_string == ' ') msg_string ++;

				if (!stricmp(msg_string, "SlitToggle"))
				{
					if(inSlit && hSlit)
					{
						// We are in the slit, so lets unload and get out..
						SendMessage(hSlit, SLIT_REMOVE, NULL, (LPARAM)hwndPlugin);

						// Here you can move to where ever you want ;)
						SetWindowPos(hwndPlugin, NULL, xpos, ypos, 0, 0, SWP_NOZORDER | SWP_NOSIZE);
						inSlit = false;
						set_window_modes();
					}
					/* Make sure before you try and load into the slit that you have
					 * the HWND of the slit ;)
					 */
					else if(hSlit)
					{
						// (Back) into the slit..
						inSlit = true;
						set_window_modes();
						SendMessage(hSlit, SLIT_ADD, NULL, (LPARAM)hwndPlugin);
					}
					
					set_setting(&inSlit,        // the * to the bool to toggle
						msg_string,                     // the message to display on toolbar
						"BBPlugManager.plugin.inSlit:"   // the rc-setting to write
						);
				}
				if (strstr(msg_string, "Edit") != NULL)
				{
					const char *temp = strchr(msg_string, ' ');
					temp++;
					char modpath[MAX_PATH], path[MAX_PATH];
					GetModuleFileName(hInstance, modpath, sizeof(modpath));
					strcpy(path, getFileDir(modpath));
					strcat(path, temp);
					strcpy(modpath, path);
					
					GetBlackboxEditor(szTemp);
					BBExecute(NULL, NULL, szTemp, path, NULL, SW_SHOWNORMAL, false);
					break;
				}
				if (!stricmp(msg_string, "AlwaysOnTop"))
				{
					// make use of our frienly ghost from above
					toggle_setting(&alwaysOnTop,        // the * to the bool to toggle
						msg_string,                     // the message to display on toolbar
						"BBPlugManager.plugin.alwaysontop:"   // the rc-setting to write
						);
					break;
				}
				if (!stricmp(msg_string, "SnapWindowToEdge"))
				{
					toggle_setting(&snapWindow,
						msg_string,
						"BBPlugManager.plugin.snapwindow:"
						);
					break;
				}
				if (!stricmp(msg_string, "TransparencyEnabled"))
				{
					toggle_setting(&transparencyEnabled,
						msg_string,
						"BBPlugManager.plugin.transparency.enabled:"
						);
					break;
				}
				if (!stricmp(msg_string, "CheckForUpdates"))
				{
					CreateThread(NULL, 0, CheckForUpdates, NULL, 0, NULL);
					break;
				}
				if (!stricmp(msg_string, "TransbackEnabled"))
				{
					toggle_setting(&transback,
						msg_string,
						"BBPlugManager.plugin.transback.enabled:"
						);
					break;
				}
				if (!stricmp(msg_string, "MenuClipperEnabled"))
				{
					toggle_setting(&clipmenu,
						msg_string,
						"BBPlugManager.plugin.menuclipper.enabled:"
						);
					if(clipmenu){
						StartMenuClipper();
					}else{
						EndMenuClipper();
					}
					break;
				}
				
				if (strstr(msg_string, "ToggleSlitAid") != NULL){
					const char *temp = strchr(msg_string, ' ');
					int n = atoi(temp);
					if(n < 0 || n >= MAX_PLUGINS) break;
					if(plugs[n].inuse && plugs[n].slitaidable){
						plugs[n].slitaid = !plugs[n].slitaid;
						MarkToggle(n, !plugs[n].loadOnStart, plugs[n].slitaid);
						UnloadPlugin(&plugs[n]);
						LoadPlugin(&plugs[n]);
						EnumWindows((WNDENUMPROC)EnumWindowTransparencyInit, NULL);
					}
					break;
				}
				
				if (strstr(msg_string, "TogglePlugin") != NULL){
					const char *temp = strchr(msg_string, ' ');
					int n = atoi(temp);
					if(n < 0 || n >= MAX_PLUGINS) break;
					if(plugs[n].inuse){
						if(plugs[n].running){
							MarkToggle(n, true, plugs[n].slitaid);
							UnloadPlugin(&plugs[n]);
						}else{
							MarkToggle(n, false, plugs[n].slitaid);
							LoadPlugin(&plugs[n]);
							EnumWindows((WNDENUMPROC)EnumWindowTransparencyInit, NULL);
						}
					}
					break;
				}
				
				if (strstr(msg_string, "RemovePlugin") != NULL){
					const char *temp = strchr(msg_string, ' ');
					int n = atoi(temp);
					if(n < 0 || n >= MAX_PLUGINS) break;
					if(plugs[n].inuse){removePlugin(n);}
					break;
				}
				
				if (strstr(msg_string, "UpdatePlugin") != NULL){
					const char *temp = strchr(msg_string, ' ');
					int n = atoi(temp);
					if(n < 0 || n >= MAX_PLUGINS) break;
					if(plugs[n].inuse){
						if(PluginToUpdate == -1){
							PluginToUpdate = n;
							CreateThread(NULL, 0, UpdatePlugin, NULL, 0, NULL);
						}else{
							MessageBox(NULL, "You must wait for current download\nto complete before starting another!", szVersion, MB_OK | MB_ICONERROR);
						}
					}
					break;
				}

				static char trans_alpha_id[] = "TransparencyAlpha ";
				if (!memicmp(msg_string, trans_alpha_id, sizeof(trans_alpha_id)-1))
				{
					transparencyAlpha = atoi(msg_string + sizeof(trans_alpha_id)-1);
					set_window_modes();
					WriteInt(rcpath, "BBPlugManager.plugin.transparency.alpha:", transparencyAlpha);
					break;
				}
				
				if (!stricmp(msg_string, "GlobalTransEnabled"))
				{
					toggle_setting(&globalTransparency,
						msg_string,
						"BBPlugManager.plugin.GlobalTransparency:"
						);
					if(globalTransparency){
						EnumWindows((WNDENUMPROC)EnumWindowTransparencyInit, NULL);
					}else{
						EnumWindows((WNDENUMPROC)EnumWindowsOpaque, NULL);
						EnumWindows((WNDENUMPROC)EnumWindowTransparencyInit, NULL);
					}
					break;
				}
				if (strstr(msg_string, "GlobalTransRem") != NULL)
				{
					EnumWindows((WNDENUMPROC)EnumWindowsOpaque, NULL);
					const char *temp = strchr(msg_string, ' ');
					int n = atoi(temp); char buffer[MAX_PATH];
					strcpy(classes[n], "");
					sprintf(buffer, "BBPlugManager.plugin.transClass%d:", n);
					WriteString(rcpath, buffer, classes[n]);
					EnumWindows((WNDENUMPROC)EnumWindowTransparencyInit, NULL);
					break;
				}
				if (strstr(msg_string, "GlobalTransAdjust") != NULL)
				{
					EnumWindows((WNDENUMPROC)EnumWindowsOpaque, NULL);
					const char *temp = strchr(msg_string, ' ');
					globalTransAlpha = atoi(temp);
					WriteInt(rcpath, "BBPlugManager.plugin.GlobTransAlpha:",globalTransAlpha);
					EnumWindows((WNDENUMPROC)EnumWindowTransparencyInit, NULL);
					break;
				}
				if (!stricmp(msg_string, "GlobalTransAdd"))
				{
					AddWindowClass_Trans();
					break;
				}
				
				if (!stricmp(msg_string, "GlobalStickyEnabled"))
				{
					toggle_setting(&globalSticky,
						msg_string,
						"BBPlugManager.plugin.GlobalSticky:"
						);
					if(globalSticky){
						EnumWindows((WNDENUMPROC)EnumWindowTransparencyInit, NULL);
					}else{
						EnumWindows((WNDENUMPROC)EnumWindowsOpaque, NULL);
						EnumWindows((WNDENUMPROC)EnumWindowTransparencyInit, NULL);
					}
					break;
				}
				if (strstr(msg_string, "GlobalStickyRem") != NULL)
				{
					EnumWindows((WNDENUMPROC)EnumWindowsOpaque, NULL);
					const char *temp = strchr(msg_string, ' ');
					int n = atoi(temp); char buffer[MAX_PATH];
					strcpy(sticky[n], "");
					sprintf(buffer, "BBPlugManager.plugin.stickyClass%d:", n);
					WriteString(rcpath, buffer, sticky[n]);
					EnumWindows((WNDENUMPROC)EnumWindowTransparencyInit, NULL);
					break;
				}
				if (!stricmp(msg_string, "GlobalStickyAdd"))
				{
					AddWindowClass_Sticky();
					break;
				}

				if (!stricmp(msg_string, "ToggleWithPlugins"))
				{
					toggle_setting(&pluginToggle,
						msg_string,
						"BBPlugManager.plugin.togglewithplugins:"
						);
					break;
				}
				if (!stricmp(msg_string, "AddPlugin"))
				{
					addPlugin();
					break;
				}
				if (!stricmp(msg_string, "AboutPlugins"))
				{
					aboutPlugins();
					break;
				}
				if (strstr(msg_string, "PluginBroams"))
				{
					const char *temp = strchr(msg_string, ' ');
					int n = atoi(temp);
					pluginBroams(n);
					break;
				}
				
				if (strstr(msg_string, "SendPluginBroam"))
				{
					const char *temp;
					temp = strstr(msg_string, "SendPluginBroam");
					temp += strlen("SendPluginBroam");
					
					char buf[MAX_PATH];
					strcpy(buf, temp);
					
					CleanString(buf);
					
					SendMessage(hwndBlackbox, BB_BROADCAST, 0, (LPARAM)buf);
					break;
				}
				
				if (!stricmp(msg_string, "About"))
				{
					sprintf(szTemp,
						"%s\n\n"
						"Author : Kaloth\n"
						"Web: http://www.geocities.com/madant8\n"
						"Email: mulletwarriorextreem@hotmail.com",
						szVersion
						);
					MessageBox(
						NULL, szTemp, "About this plugin...",
						MB_OK | MB_ICONINFORMATION);
					break;
				}
			}
			break;
		}
		// ==========

		case WM_NCRBUTTONUP:
			ShowMyMenu(true);
			break;

		// ==========
		// The window is moving
		case WM_WINDOWPOSCHANGING:
			// Is SnapWindowToEdge enabled? - and the plugin not hidden?
			if (!inSlit && snapWindow && IsWindowVisible(hwnd))
			{
				// ...then snap window to screen edges, if the distance is
				// less than 10 pixel (if the last bool is false it uses the
				// current DesktopArea)
				SnapWindowToEdge((WINDOWPOS*)lParam, 10, true);
			}
			break;

		// ==========
		// Save window position if it changed...
		case WM_WINDOWPOSCHANGED:
		{
			if(!inSlit){
				WINDOWPOS* windowpos = (WINDOWPOS*)lParam;
				xpos = windowpos->x;
				ypos = windowpos->y;
				WriteInt(rcpath, "BBPlugManager.plugin.x:", xpos);
				WriteInt(rcpath, "BBPlugManager.plugin.y:", ypos);
			}
			break;
		}

		// Allow window to move...
		case WM_NCHITTEST:
			if(!inSlit){
				return HTCAPTION;
			}
			return HTCLIENT;

		// Prevent closing with alt-f4
		case WM_CLOSE:
			break;
		
		case WM_RBUTTONUP:
			ShowMyMenu(true);
			break;
		// Let windows do everything else
		default:
			return DefWindowProc(hwnd,message,wParam,lParam);
	}
	// return zero, except from the hit-test message
	return 0;
}

void GetStyleSettings()
{
	if (hFont)
		DeleteObject(hFont), hFont = NULL;

	myStyleItem = *(StyleItem*)GetSettingPtr(SN_TOOLBAR);

	bevelWidth  = *(int*)GetSettingPtr(SN_BEVELWIDTH);
	borderWidth = *(int*)GetSettingPtr(SN_BORDERWIDTH);
	borderColor = *(COLORREF*)GetSettingPtr(SN_BORDERCOLOR);

	hFont = CreateStyleFont(&myStyleItem);
}
